import React, { Component, PropTypes } from 'react';
import ReactDOM from 'react-dom';
import {
  createStore,
  combineReducers,
  applyMiddleware
} from 'redux';
import { connect, Provider } from 'react-redux';
import {
  Form,
  Control,
  Errors,
  actions,
  combineForms,
} from 'react-redux-form';
import thunk from 'redux-thunk';
import createLogger from 'redux-logger';

const isEmpty_ = (val) => !val;

const store = createStore(combineForms({
  user: {
    username: '',
    items: []
  },
}), applyMiddleware(thunk, createLogger()));


const INITIAL_ITEM = {name: '', subitems: []};
const INITIAL_SUBITEM = {name: ''};
const isRequired = value => !isEmpty_(value);


class App extends Component {
  constructor(props) {
    super(props);
    
    this.onItemAdd = this.onItemAdd.bind(this);
    this.onSubItemAdd = this.onSubItemAdd.bind(this);
    this.onItemDelete = this.onItemDelete.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
  }
  
  onItemDelete(itemIndex) {
    const { dispatch } = this.props;
    if (confirm("Delete?")) {
      dispatch(actions.remove('user.items', itemIndex));
    }
  }
  
  onItemAdd() {
    const { dispatch } = this.props;
    dispatch(actions.push('user.items', INITIAL_ITEM));
  }
  
  onSubmit(data) {
    console.error("################ handleSubmit(): %O ################", data);
  }
  
  onSubItemAdd(itemIndex) {
    const { dispatch } = this.props;
    dispatch(actions.push(`user.items[${itemIndex}].subitems`, INITIAL_SUBITEM));
  }

  handleSubmit() {
    alert('HI');
  }
  
  render() {
    // const { user } = this.props;
    
    return (
      <Form
        model="user"
        onSubmit={this.handleSubmit}
        validateOn="submit"
      >
        <div>
          <Control
            model=".email"

            type="text"
            placeholder="Email address"
            autoCorrect="off"
            autoCapitalize="off"
            spellCheck="false"
            validators={{
              email: (v) => v && !!~v.indexOf('@'),
            }}
          />
          <Errors
            model=".email"
            messages={{
              required: 'Required',
              email: 'Must be a valid email address',
            }}
          />
        </div>
        <div >
          <Control
            model=".password"
            type="password"
            placeholder="Password"
            validators={{
              password: (v) => v && v.length > 2,
            }}
          />
          <Errors
            model=".password"
            messages={{
              required: 'Required',
              password: 'Weak password',
            }}
          />
        </div>
        <div>
          <button>Submit</button>
        </div>
      </Form>
    );
  }
}

const mapStateToProps = state => ({
  user: state.user
});

const BoundApp = connect(mapStateToProps)(App);

ReactDOM.render(
  <Provider store={store}>
    <App />
  </Provider>
, document.querySelector('#app'));
